@@ -163,7 +163,7 @@ class AxlToolkit:
     """
 
     def __init__(self, username, password, server_ip, version='12.5', tls_verify=True, timeout=10,
-                 logging_enabled=False, schema_folder_path=None):
+                 logging_enabled=False):
         """
         Constructor - Create new instance
         """
@@ -175,32 +175,30 @@ class AxlToolkit:
         self.wsdl = None
         self.last_exception = None
 
-        filedir = os.path.dirname(__file__)
-        if schema_folder_path is not None:
-            filedir = schema_folder_path
+        schema_folder_path = os.path.dirname(__file__)
 
         self.cache = SqliteCache(path='/tmp/sqlite_axl_{0}.db'.format(server_ip), timeout=60)
 
         if version == '14.0':
-            self.wsdl = os.path.join(filedir, 'schema/14.0/AXLAPI.wsdl')
+            self.wsdl = os.path.join(filedir, f'schema/{version}/AXLAPI.wsdl')
         elif version == '12.5':
-            self.wsdl = os.path.join(filedir, 'schema/12.5/AXLAPI.wsdl')
+            self.wsdl = os.path.join(filedir, f'schema/{version}/AXLAPI.wsdl')
         elif version == '12.0':
-            self.wsdl = os.path.join(filedir, 'schema/12.0/AXLAPI.wsdl')
+            self.wsdl = os.path.join(filedir, f'schema/{version}/AXLAPI.wsdl')
         elif version == '11.5':
-            self.wsdl = os.path.join(filedir, 'schema/11.5/AXLAPI.wsdl')
+            self.wsdl = os.path.join(filedir, f'schema/{version}/AXLAPI.wsdl')
         elif version == '11.0':
-            self.wsdl = os.path.join(filedir, 'schema/11.0/AXLAPI.wsdl')
+            self.wsdl = os.path.join(filedir, f'schema/{version}/AXLAPI.wsdl')
         elif version == '10.5':
-            self.wsdl = os.path.join(filedir, 'schema/10.5/AXLAPI.wsdl')
+            self.wsdl = os.path.join(filedir, f'schema/{version}/AXLAPI.wsdl')
         elif version == '10.0':
-            self.wsdl = os.path.join(filedir, 'schema/10.0/AXLAPI.wsdl')
+            self.wsdl = os.path.join(filedir, f'schema/{version}/AXLAPI.wsdl')
         else:
             self.wsdl = os.path.join(filedir, 'schema/12.5/AXLAPI.wsdl')
 
         self.client = Client(wsdl=self.wsdl, plugins=[self.history],
                              transport=Transport(timeout=timeout, operation_timeout=timeout,
-                             cache=self.cache, session=self.session))
+                                                 cache=self.cache, session=self.session))
 
         # Update the Default SOAP API Binding Address Location with server_ip for all API Service Endpoints
         # Default: (https://CCMSERVERNAME:8443/axl/)
@@ -214,7 +212,8 @@ class AxlToolkit:
         if ConnectivityTest.status_code == 200:
             pass  # Connection was successful.
         elif ConnectivityTest.status_code == 401:
-            raise ValueError('Unable to authenticate using the specified credentials and/or ip address. Please verify the information and/or reachability of the server.')
+            raise ValueError(
+                'Unable to authenticate using the specified credentials and/or ip address. Please verify the information and/or reachability of the server.')
         else:
             print(ConnectivityTest.status_code)
 
@@ -367,15 +366,16 @@ class CUCMAxlToolkit(AxlToolkit):
         else:
             schema_folder_path += "/CUCM/"
 
-
         # Create a super class, where the CUCMAxlToolkit class inherits from the AxlToolkit class.
         # This enables us to extend the parent class AxlToolkit with CUCM AXL API specic methods
         # Reference:  https://realpython.com/python-super/
         super().__init__(username, password, server_ip, version=version, tls_verify=tls_verify, timeout=timeout,
                          logging_enabled=logging_enabled, schema_folder_path=schema_folder_path)
+
     """
     UCM Group
     """
+
     def get_ucm_group(self, name):
         try:
             result = self.service.getCallManagerGroup(name=name)
@@ -413,7 +413,7 @@ class CUCMAxlToolkit(AxlToolkit):
 
         try:
             result = self.service.addCallManagerGroup(callManagerGroup={
-                                                      'name': name, 'members': {'member': member_data}})
+                'name': name, 'members': {'member': member_data}})
         except Exception as fault:
             result = None
             self.last_exception = fault
@@ -2025,7 +2025,8 @@ class UcmRisPortToolkit:
         # Update the Default SOAP API Binding Address Location with server_ip for all API Service Endpoints
         # Default: (https://localhost:8443/realtimeservice2/services/RISService70)
         self.service = self.client.create_service("{http://schemas.cisco.com/ast/soap}RisBinding",
-                                                  "https://{0}:8443/realtimeservice2/services/RISService70".format(server_ip))
+                                                  "https://{0}:8443/realtimeservice2/services/RISService70".format(
+                                                      server_ip))
 
         if logging_enabled:
             AxlToolkit._enable_logging()
@@ -2076,7 +2077,8 @@ class UcmPerfMonToolkit:
         # Update the Default SOAP API Binding Address Location with server_ip for all API Service Endpoints
         # Default: (https://localhost:8443/perfmonservice2/services/PerfmonService)
         self.service = self.client.create_service("{http://schemas.cisco.com/ast/soap}PerfmonBinding",
-                                                  "https://{0}:8443/perfmonservice2/services/PerfmonService".format(server_ip))
+                                                  "https://{0}:8443/perfmonservice2/services/PerfmonService".format(
+                                                      server_ip))
 
         if logging_enabled:
             AxlToolkit._enable_logging()
@@ -2166,7 +2168,8 @@ class UcmLogCollectionToolkit:
         self.session.auth = HTTPBasicAuth(username, password)
         self.session.verify = tls_verify
         self.history = AXLHistoryPlugin(maxlen=1)
-        self.wsdl = 'https://{0}:8443/logcollectionservice2/services/LogCollectionPortTypeService?wsdl'.format(server_ip)
+        self.wsdl = 'https://{0}:8443/logcollectionservice2/services/LogCollectionPortTypeService?wsdl'.format(
+            server_ip)
         self.last_exception = None
 
         self.cache = SqliteCache(path='/tmp/sqlite_logcollection.db', timeout=60)
@@ -2179,7 +2182,8 @@ class UcmLogCollectionToolkit:
         # Update the Default SOAP API Binding Address Location with server_ip for all API Service Endpoints
         # Default: (https://localhost:8443/logcollectionservice2/services/LogCollectionPortTypeService)
         self.service = self.client.create_service("{http://schemas.cisco.com/ast/soap}LogCollectionPortSoapBinding",
-                                                  "https://{0}:8443/logcollectionservice2/services/LogCollectionPortTypeService".format(server_ip))
+                                                  "https://{0}:8443/logcollectionservice2/services/LogCollectionPortTypeService".format(
+                                                      server_ip))
 
         if logging_enabled:
             AxlToolkit._enable_logging()
@@ -2234,3 +2238,71 @@ class UcmDimeGetFileToolkit:
 
     def get_service(self):
         return self.service
+
+
+class remotePhoneAccess:
+    """
+    Reference: https://web.archive.org/web/20220119230219/https://www.cisco.com/c/en/us/td/docs/voice_ip_comm/cuipph/all_models/xsi/9-1-1/CUIP_BK_P82B3B16_00_phones-services-application-development-notes/CUIP_BK_P82B3B16_00_phones-services-application-development-notes_chapter_0101.html
+    """
+
+    def __init__(self, phone_ip, axlUsername, axlPassword, server_ip):
+        self.phone_ip = phone_ip
+        self.axlUsername = axlUsername
+        self.axlPassword = axlPassword
+        self.server_ip = server_ip
+        ConnectivityTest = (self.session.get(f'https://{self.axlUsername}:{self.axlPassword}@{self.phone_ip}'))
+        if ConnectivityTest.status_code == 200:
+            logging.info(f'Successfully connected to Phone: {self.phone_ip}')
+        elif ConnectivityTest.status_code == 401:
+            raise Exception(
+                'Unable to authenticate using the specified credentials and/or ip address. Please verify the information and/or reachability of the server.')
+
+    def callmanager_setup(self):
+        """
+        Accesses the phone's webpage to get the MAC address. Then updates the application user specified to include the phone's MAC address so it can be controlled remotely.
+        :return:
+        """
+        CUCMAXL = CUCMAxlToolkit(username=self.axlUsername, password=self.axlPassword, server_ip=self.server_ip)
+        phoneWebpageInfo = requests.get(f"http://{self.phone_ip}", verify=False).text
+        rex = re.compile('SEP............')
+        macAddress = rex.search(phoneWebpageInfo).group()
+        appUser = CUCMAXL.get_app_user(userid=self.axlUsername)
+        associatedDevices = (appUser['return']['appUser']['associatedDevices'])
+        if associatedDevices is not None:
+            associatedDevices = associatedDevices['device']
+            if macAddress not in associatedDevices:
+                associatedDevices.append(macAddress)
+                CUCMAXL.update_app_user(userid=self.axlUsername, associatedDevices={'device': associatedDevices})
+
+    def get_phone_screenshot(self):
+        """
+        Returns the bytes of the screenshot url request.
+        :return:
+        """
+        url = f'https://{self.axlUsername}:{self.axlPassword}@{self.phone_ip}/CGI/Screenshot'
+
+        r = requests.get(url, allow_redirects=True, verify=False, stream=True).content
+        return r
+
+    # Define our function to iterate through our key press list
+    # and send the key press to the target device
+    def send_key_press(self, keyPress):
+        """
+        Sends button presses to the phone IP specified. Example usage: send_key_press({"Key:KeyPad0"})
+        :param keyPress: Key to send to the phone.
+        :return:
+        """
+        messages = []
+        d = OrderedDict(
+            [("CiscoIPPhoneExecute", OrderedDict(
+                [("ExecuteItem", OrderedDict([("@Priority", "0"), ("@URL", keyPress)])
+                  )])
+              )])
+        messages.append({"XML": unparse(d)})
+        headers = {"content-type": "application/xml"}
+        for key_press in messages:
+            r = requests.post(
+                url="http://" + self.phone_ip + "/CGI/Execute",
+                data=key_press, verify=False, headers=headers, auth=(self.axlUsername, self.axlPassword)
+            )
+            logging.info(r.text)
